@@ -46,6 +46,7 @@
 #include <signal.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <fcntl.h>
 
 #include <libgnome/libgnome.h>
 
@@ -118,20 +119,20 @@ static int
 pilot_connect (GPilotDevice *device,int *error) 
 {
 	struct pi_sockaddr addr;
-	int sd;
+	int sd, listen_sd;
 	int ret;
     
 	if (device->type != PILOT_DEVICE_NETWORK) {
 		pilot_set_baud_rate (device);
 	}
 
-	if (!(sd = pi_socket (PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
+	if (!(listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP))) {
 		g_warning ("pi_socket: %s",strerror (errno));
 		if (error) *error = 1;
 		return -1;
 	}
 	
-	addr.pi_family = PI_AF_SLP;
+	addr.pi_family = PI_AF_PILOT;
 
 /*
 	Most important for networking
@@ -145,29 +146,38 @@ pilot_connect (GPilotDevice *device,int *error)
 
 	strcpy (addr.pi_device,device->port);
 
-	ret = pi_bind (sd, (struct sockaddr*)&addr, sizeof (addr));
+	ret = pi_bind (listen_sd, (struct sockaddr*)&addr, sizeof (addr));
 	if (ret == -1) {
 		g_warning (_("Unable to bind to pilot"));
-		if (error) *error = 1;
+		if (error)
+			*error = 1;
+		pi_close(listen_sd);
 		return 0;
 	}
 
-	ret = pi_listen (sd,1);
+	ret = pi_listen (listen_sd, 1);
 	if (ret != 0) {
 		g_warning ("pi_listen: %s", strerror (errno));
-		if (error) *error = 2;
+		if (error)
+			*error = 2;
+		pi_close(listen_sd);
 		return 0;
 	}
 
-	sd = pi_accept_to (sd, NULL,0, device->timeout); 
+	sd = pi_accept_to (listen_sd, NULL,0, device->timeout); 
 	if (sd == -1) {
 		g_warning ("pi_accept_to: %s", strerror (errno));
 		g_warning ("pi_accept_to: timeout was %d secs", device->timeout);
-		if (error) *error = 2;
+		if (error)
+			*error = 2;
+		pi_close(listen_sd);
 		return 0;
 	}
 
-	if (error) *error = 0;
+	if (error)
+		*error = 0;
+	pi_close(listen_sd);
+	
 	return sd;
 }
 
@@ -634,7 +644,6 @@ sync_device (GPilotDevice *device, GPilotContext *context)
 			}
 		}
 		pilot_disconnect (pfd);
-		close (pfd);
 	} else {
 		if (connect_error==1) return FALSE; /* remove this device */
 		else return TRUE;
@@ -803,12 +812,21 @@ network_device_err (GIOChannel *io_channel,
 
 #ifdef WITH_USB_VISOR
 #ifdef linux
+static const char *product_names[] =
+{
+	"Handspring Visor",
+	"Palm Handheld",
+	NULL
+};
+
+
 static gboolean 
 visor_devices_in (GIOChannel *io_channel,
 		 GIOCondition condition,
 		 GPilotContext *context) {
 	GPilotDevice *device;
-	GList *i;
+	GList *l;
+	int i;
 	gboolean visor_exists = FALSE;
 	char line[256]; /* this is more than enough to fit any line from 
 			 * /proc/bus/usb/devices */
@@ -824,32 +842,32 @@ visor_devices_in (GIOChannel *io_channel,
 
 	/* Check /proc/bus/usb/devices for a visor */
 	f = fopen ("/proc/bus/usb/devices", "r");
-
-	fgets (line, 255, f);
-	while (!feof (f)) {
-		if (line[0] == 'S') {
-			if (!strncmp (line + 4, "Product=Handspring Visor",
-				      24)) {
+	
+	while (fgets (line, 255, f) != NULL && !visor_exists) {
+		if (line[0] != 'S')
+			continue;
+		
+		for (i = 0; product_names[i] != NULL; i++) {
+			if (!strncmp (line + strlen ("S:  Product="), 
+				      product_names[i], 
+				      strlen (product_names[i])))
 				visor_exists = TRUE;
-				break;
-			}
 		}
-		fgets (line, 255, f);
 	}
 	
 	fclose (f);
 	
 	if (visor_exists) {
-		i = context->devices;
-		while (i) {
-			device = i->data;
+		l = context->devices;
+		while (l) {
+			device = l->data;
 			if (device->type == PILOT_DEVICE_USB_VISOR) {
 				/* just try to synch.  Until I can talk to 
 				 * the kernel guys this is the best way to 
                                  * go. */
 				sync_device (device, context);
 			}
-			i = i->next;
+			l = l->next;
 		}
 	}
 
